/// <reference types="node" />
import { Stream } from 'stream';
import { ViewResult, FileResult } from './restults';
import { RedirectResult } from './index';
/**
 * Base Controller.
 *
 * @export
 * @class Controller
 */
export declare abstract class BaseController {
    view(viewName: string, model?: object): ViewResult;
    file(file: string | Buffer | Stream, contentType?: string, fileDownloadName?: string): FileResult;
    /**
     * Perform a 302 redirect to `url`.
     *
     * The string "back" is special-cased
     * to provide Referrer support, when Referrer
     * is not present `alt` or "/" is used.
     *
     * Examples:
     *
     *    this.redirect('back');
     *    this.redirect('back', '/index.html');
     *    this.redirect('/login');
     *    this.redirect('http://google.com');
     * @param {string} url
     * @param {string} [alt]
     * @returns
     * @memberof BaseController
     */
    redirect(url: string, alt?: string): RedirectResult;
}
