"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BaseRoute_1 = require("./BaseRoute");
const tsioc_1 = require("tsioc");
const decorators_1 = require("../decorators");
const decorators_2 = require("../decorators");
const util_1 = require("../util");
const index_1 = require("../errors/index");
const util_2 = require("util");
const restults_1 = require("../restults");
class ControllerRoute extends BaseRoute_1.BaseRoute {
    constructor(route, controller) {
        super(route);
        this.controller = controller;
    }
    async navigating(container, ctx) {
        try {
            switch (ctx.method) {
                case 'GET':
                    await this.invoke(ctx, container, decorators_1.Get, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
                case 'POST':
                    await this.invoke(ctx, container, decorators_1.Post, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
                case 'Put':
                    await this.invoke(ctx, container, decorators_1.Put, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
                case 'DElETE':
                    await this.invoke(ctx, container, decorators_1.Delete, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
            }
        }
        catch (err) {
            if (err instanceof index_1.HttpError) {
                ctx.status = err.code;
                ctx.response.status = err.code;
                ctx.response.message = err.message;
            }
            console.log(err.toString());
        }
    }
    createProvider(container, meta, params, ctrl, ctx) {
        if (params && params.length) {
            let paramVal = null;
            if (this.isRestUri(meta.route)) {
                let route = meta.route.substr(0, meta.route.indexOf('/:')) + '/';
                let baseURL = this.cutEmptyPath(this.url, true);
                let routeUrl = this.cutEmptyPath(ctx.url.replace(baseURL, ''));
                let querystring = routeUrl.replace(route, '');
                paramVal = querystring.indexOf('/') > 0 ? querystring.substr(0, querystring.indexOf('/')) : querystring;
            }
            let body = ctx.request['body'] || {};
            console.log('request body:', body);
            let providers = params.map((p, idx) => {
                try {
                    if (tsioc_1.isClass(p) && !this.isBaseType(p)) {
                        let meta = tsioc_1.getPropertyMetadata(decorators_1.Field, p);
                        let val = container.get(p);
                        for (let n in meta) {
                            if (!util_2.isUndefined(body[n])) {
                                val[n] = body[n];
                            }
                        }
                        console.log('result model:', val);
                        return {
                            value: val,
                            index: idx
                        };
                    }
                    else if (this.isBaseType(p)) {
                        let val;
                        if (paramVal !== null) {
                            if (p === String) {
                                val = paramVal;
                            }
                            else if (p === Boolean) {
                                val = new Boolean(paramVal);
                            }
                            else if (p === Number) {
                                val = parseFloat(paramVal);
                            }
                            else if (p === Date) {
                                val = new Date(paramVal);
                            }
                            paramVal = null;
                        }
                        return {
                            value: val,
                            index: idx
                        };
                    }
                    else {
                        return null;
                    }
                }
                catch (err) {
                    throw new index_1.BadRequestError(err.toString());
                }
            });
            return providers.filter(p => p !== null);
        }
        return [];
    }
    isBaseType(p) {
        if (!tsioc_1.isToken(p)) {
            return true;
        }
        if (p === Boolean || p === String || p === Number) {
            return true;
        }
        if (!tsioc_1.isFunction(p)) {
            return true;
        }
        return false;
    }
    isRestUri(uri) {
        return /\/:/.test(uri || '');
    }
    async invoke(ctx, container, decorator, provider) {
        let decoratorName = decorator.toString();
        let baseURL = this.cutEmptyPath(this.url, true);
        let routPath = this.cutEmptyPath(ctx.url.replace(baseURL, ''));
        let methodMaps = tsioc_1.getMethodMetadata(decoratorName, this.controller);
        let meta;
        let allMethods = [];
        for (let name in methodMaps) {
            allMethods = allMethods.concat(methodMaps[name]);
        }
        allMethods = allMethods.sort((ra, rb) => (rb.route || '').length - (ra.route || '').length);
        meta = allMethods.find(route => (route.route || '') === routPath);
        if (!meta) {
            meta = allMethods.find(route => {
                let uri = route.route || '';
                // if (uri === routPath) {
                //     return true;
                // }
                if (this.isRestUri(uri)) {
                    let idex = uri.indexOf('/:');
                    let url = uri.substr(0, idex);
                    if (url !== routPath && routPath.indexOf(url) === 0) {
                        return true;
                    }
                }
                return false;
            });
        }
        if (meta && meta.propertyKey) {
            let ctrl = container.get(this.controller);
            if (container.has(util_1.symbols.IAuthorization)) {
                let hasAuth = Reflect.hasMetadata(decorators_2.Authorization.toString(), ctrl, meta.propertyKey);
                if (hasAuth) {
                    let auth = container.get(util_1.symbols.IAuthorization);
                    if (!auth.isAuth()) {
                        throw new index_1.UnauthorizedError();
                    }
                }
            }
            let params = container.getMethodParameters(this.controller, ctrl, meta.propertyKey);
            let response = await container.invoke(this.controller, meta.propertyKey, ctrl, ...provider(meta, params, ctrl));
            let contentType = meta.contentType;
            if (util_2.isString(response)) {
                contentType = contentType || 'text/html';
                ctx.response.body = response;
            }
            else if (util_2.isArray(response) && response instanceof Uint8Array) {
                ctx.response.body = Buffer.from(response);
            }
            else if (util_2.isObject(response)) {
                if (response instanceof restults_1.ResultValue) {
                    await response.sendValue(ctx, container);
                }
                else {
                    contentType = contentType || 'application/json';
                    ctx.type = contentType;
                    ctx.response.body = response;
                    ctx.redirect;
                }
            }
        }
        else {
            throw new index_1.NotFoundError();
        }
    }
}
exports.ControllerRoute = ControllerRoute;

//# sourceMappingURL=../sourcemaps/router/ControllerRoute.js.map
